/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package wsdl;

/**
 * Junit test to test using ClassLoader to find wsdl and more importantly
 * imports in the wsdl
 * 
 * -----------------------------------------------------------------
 * NOTE: YOU MUST HAVE THE ClassloaderTest.jar ON YOUR CLASSPATH TO
 *       BE ABLE TO RUN THIS TEST
 * -----------------------------------------------------------------
 * 
 * @author Owen Burroughs
 */

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;

import javax.wsdl.Definition;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;
import org.apache.wsif.util.WSIFUtils;
import util.TestUtilities;

public class ClassLoaderTest extends TestCase {
    String wsdlLocation = "test/ImportingTest.wsdl";
    String wsdlLocation2 = "test/ImportingTest2.wsdl";
    String wsdlLocation3 = "test/ImportingTest3.wsdl";
    String wsdlLocation4 = "ImportingTest4.wsdl";
    String wsdlLocation5 = "test/subtest/ImportingTest5.wsdl";
    String wsdlLocation6 = "test/subtest/subtest2/ImportingTest6.wsdl";         
    String wsdlLocation7 =
        TestUtilities.getWsdlPath("java\\test\\addressbook")
            + "ImportingAddressBook.wsdl";
    String wsdlLocationError = "test/subtest/subtest2/ImportingTestError.wsdl";
    String wsdlLocationA = "ImportingTestA.wsdl";              

    public ClassLoaderTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        return new TestSuite(ClassLoaderTest.class);
    }

    public void setUp() {
        TestUtilities.setUpExtensionsAndProviders();
    }

    // wsdl location	= test/ImportingTest.wsdl
    // import location	= AddressBookTest.wsdl
    public void test() {
        doIt(wsdlLocation);
    }

    // wsdl location	= test/ImportingTest2.wsdl
    // import location	= /test/subtest/subtest2/AddressBookTest2.wsdl
    public void test2() {
        doIt(wsdlLocation2);
    }

    // wsdl location	= test/ImportingTest3.wsdl
    // import location	= subtest/subtest2/AddressBookTest3.wsdl
    public void test3() {
        doIt(wsdlLocation3);
    }

    // wsdl location	= ImportingTest4.wsdl
    // import location	= test/subtest/subtest2/AddressBookTest4.wsdl
    public void test4() {
        doIt(wsdlLocation4);
    }

    // wsdl location	= test/subtest/ImportingTest5.wsdl
    // import location	= ../AddressBookTest.wsdl
    public void test5() {
        doIt(wsdlLocation5);
    }

    // wsdl location	= test/subtest/subtest2/ImportingTest6.wsdl
    // import location	= ../../AddressBookTest.wsdl
    public void test6() {
        doIt(wsdlLocation6);
    }

    // wsdl location	= <wsdlPath>/java/test/addressbook/ImportingAddressBook.wsdl
    // import location	= http://localhost:8080/wsdl/AddressBook.wsdl
    public void test7() {
        if (TestUtilities.areWeTesting("remotewsdl")) {
            doIt(wsdlLocation7);
        }
    }
    
    public void test8() {
    	doItWithReader(wsdlLocation2);
    }

    // wsdl location	= ImportingTestA.wsdl
    // import location	= /test/ImportingTest.wsdl
    // import location2	= AddressBookTest.wsdl
    public void test9() {
    	doIt(wsdlLocationA);
    }

    // wsdl location	= test/subtest/subtest2/ImportingTestError.wsdl
    // import location	= ../../../../AddressBookTest.wsdl
    public void testError() {
    	doItError(wsdlLocationError);
    }

    private void doIt(String wsdlLoc) {
        try {
            WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
            WSIFService service =
                factory
                    .getService(wsdlLoc, this.getClass().getClassLoader(), null,
                                // serviceNS
                                null, // serviceName
                                "http://wsifservice.addressbook/", // portTypeNS
                                "AddressBook"); // portTypeName
        } catch (Exception e) {
            System.err.println(
                "ClassLoaderTest(" + wsdlLoc + ") caught exception " + e);
            e.printStackTrace();
            assertTrue(false);
        }
    }

    private void doItError(String wsdlLoc) {
        try {
            WSIFServiceFactory factory = WSIFServiceFactory.newInstance();
            WSIFService service =
                factory
                    .getService(wsdlLoc, this.getClass().getClassLoader(), null,
                                // serviceNS
                                null, // serviceName
                               "http://wsifservice.addressbook/", // portTypeNS
                               "AddressBook"); // portTypeName
    		assertTrue("Import location of '../../../../AddressBook.wsdl'" +
    					"should have caused an error", false);
        } catch (Exception e) {
        	// Excpected so ignore
        }
    }

    private void doItWithReader(String wsdlLoc) {
        InputStream in = null;
        ClassLoader loader = this.getClass().getClassLoader();
        try {
            URL url = null;
            if (wsdlLoc.indexOf(":") == -1)
                url = new URL("file", null, wsdlLoc);
            else
                url = new URL(wsdlLoc);
            String wsdlRelativeLocation = url.getPath();
            if (wsdlRelativeLocation.startsWith("/"))
                wsdlRelativeLocation = wsdlRelativeLocation.substring(1);
            in = loader.getResourceAsStream(wsdlRelativeLocation);
            Reader reader = new InputStreamReader(in);
            Definition def = WSIFUtils.readWSDL(url, reader, loader);
            assertNotNull("Definition is null!!", def);
        } catch (Exception e) {
            System.err.println(
                "ClassLoaderTest(" + wsdlLoc + ") caught exception " + e);
            assertTrue(false);
        }
    }
}
